@@ -924,6 +924,7 @@ read_stream_connection_unbuffered(fd, buf0, min_len, max_len)
   struct timeval	tv;
   time_t		now, then;
 
+  int		 	select_status;
  
 #if 0
   fprintf(stderr, "read_stream_connection(%d, 0x%x, %d, %d)\n",
@@ -1024,12 +1025,19 @@ read_stream_connection_unbuffered(fd, buf0, min_len, max_len)
 	{
           now = time(NULL);
 	  tv.tv_sec = INCR_TIMEOUT; tv.tv_usec = 0;
-	  if (select(realfd+1, &fdr, &fdw, NULL, &tv) > 0)
+	  select_status = select ( realfd + 1, &fdr, &fdw, NULL, &tv );
+          if ( select_status == 0 )
+          {
+      	   FD_ZERO(&fdr); FD_ZERO(&fdw);
+      	   FD_SET(realfd, &fdr); FD_SET(realfd, &fdw); 
+          }
+	  else
+	  if ( select_status > 0 )
 	    {
 	  ret = SSL_read(fp->ssl, buf + total, max_len - total);
 	  if (ret > 0)
 		{
-	    total += ret;
+	          total += ret;
 		  FD_SET(realfd, &fdr);
 		  FD_SET(realfd, &fdw); 
 		}
@@ -1039,7 +1047,8 @@ read_stream_connection_unbuffered(fd, buf0, min_len, max_len)
 	      if (ret <= 0)
 		{
 		  err = SSL_get_error(fp->ssl, ret);
-		  FD_ZERO(&fdr); FD_ZERO(&fdw);
+		  FD_ZERO(&fdr); 
+		  FD_ZERO(&fdw);
 		  switch (err)
 	   {
 		    case SSL_ERROR_WANT_READ:
@@ -1052,6 +1061,7 @@ read_stream_connection_unbuffered(fd, buf0, min_len, max_len)
 #if DEBUG_SSL > 2
 		      fprintf(stderr, "SSL_Connect[%d]: SSL_ERROR_WANT_WRITE\n", getpid());
 #endif
+		      FD_SET(realfd, &fdr);
 		      FD_SET(realfd, &fdw);
 		      break;
 
@@ -1391,6 +1401,7 @@ nrecv (fd, data, length, i_opt)
  return e;
 }
  
+
 ExtFunc int
 close_stream_connection(fd)
  int fd;
